@@ -1,7 +1,7 @@
 "DiskCache: disk and file backed cache."
 
 from .core import Cache, Disk, UnknownFileWarning, EmptyDirWarning, Timeout
-from .core import LIMITS, DEFAULT_SETTINGS, EVICTION_POLICY
+from .core import DEFAULT_SETTINGS, EVICTION_POLICY
 from .fanout import FanoutCache
 from .persistent import Deque, Index
 
@@ -11,7 +11,6 @@ __all__ = [
     'UnknownFileWarning',
     'EmptyDirWarning',
     'Timeout',
-    'LIMITS',
     'DEFAULT_SETTINGS',
     'EVICTION_POLICY',
     'FanoutCache',
@@ -28,8 +27,8 @@ except Exception:  # pylint: disable=broad-except
 
 
 __title__ = 'diskcache'
-__version__ = '2.5.2'
-__build__ = 0x020502
+__version__ = '2.7.0'
+__build__ = 0x020700
 __author__ = 'Grant Jenks'
 __license__ = 'Apache 2.0'
 __copyright__ = 'Copyright 2016 Grant Jenks'
