@@ -6,6 +6,7 @@ import operator as op
 import sys
 
 from collections import MutableMapping, OrderedDict, Sequence
+from collections import KeysView, ValuesView, ItemsView
 from itertools import islice
 from shutil import rmtree
 from tempfile import mkdtemp
@@ -55,19 +56,26 @@ class Deque(Sequence):
     Items are serialized to disk. Deque may be initialized from directory path
     where items are stored.
 
-    >>> deque = Deque()
-    >>> for value in range(5):
-    ...     deque.append(value)
+    >>> deque = Deque(directory='/tmp/diskcache/deque')
+    >>> deque
+    Deque(directory='/tmp/diskcache/deque')
+    >>> deque.clear()
+    >>> deque += range(5)
     >>> list(deque)
     [0, 1, 2, 3, 4]
     >>> for value in range(5):
     ...     deque.appendleft(-value)
+    >>> len(deque)
+    10
     >>> list(deque)
     [-4, -3, -2, -1, 0, 0, 1, 2, 3, 4]
     >>> deque.pop()
     4
     >>> deque.popleft()
     -4
+    >>> deque.reverse()
+    >>> list(deque)
+    [3, 2, 1, 0, 0, -1, -2, -3]
 
     """
     def __init__(self, iterable=(), directory=None):
@@ -86,6 +94,32 @@ class Deque(Sequence):
         self.extend(iterable)
 
 
+    @classmethod
+    def fromcache(cls, cache, iterable=()):
+        """Initialize deque using `cache`.
+
+        >>> cache = Cache('/tmp/diskcache/index')
+        >>> _ = cache.clear()
+        >>> deque = Deque.fromcache(cache, [5, 6, 7, 8])
+        >>> len(deque)
+        4
+        >>> 7 in deque
+        True
+        >>> deque.popleft()
+        5
+
+        :param Cache cache: cache to use
+        :param iterable: iterable of items
+        :return: initialized Deque
+
+        """
+        # pylint: disable=no-member,protected-access
+        self = cls.__new__(cls)
+        self._cache = cache
+        self.extend(iterable)
+        return self
+
+
     @property
     def directory(self):
         "Directory path where deque is stored."
@@ -215,10 +249,6 @@ class Deque(Sequence):
 
         Return string with printable representation of deque.
 
-        >>> deque = Deque(directory='/tmp/diskcache/deque')
-        >>> deque
-        Deque(directory='/tmp/diskcache/deque')
-
         """
         name = type(self).__name__
         return '{0}(directory={1!r})'.format(name, self.directory)
@@ -237,12 +267,6 @@ class Deque(Sequence):
 
         Extend back side of deque with items from iterable.
 
-        >>> deque = Deque(directory='/tmp/diskcache/deque')
-        >>> deque.clear()
-        >>> deque += 'abc'
-        >>> list(deque)
-        ['a', 'b', 'c']
-
         """
         self.extend(iterable)
         return self
@@ -253,15 +277,6 @@ class Deque(Sequence):
 
         Return iterator of deque from front to back.
 
-        >>> deque = Deque(directory='/tmp/diskcache/deque')
-        >>> deque.clear()
-        >>> deque += 'abc'
-        >>> iterator = iter(deque)
-        >>> next(iterator)
-        'a'
-        >>> list(iterator)
-        ['b', 'c']
-
         """
         _cache = self._cache
 
@@ -277,14 +292,6 @@ class Deque(Sequence):
 
         Return length of deque.
 
-        >>> deque = Deque(directory='/tmp/diskcache/deque')
-        >>> deque.clear()
-        >>> len(deque)
-        0
-        >>> deque.extend('abc')
-        >>> len(deque)
-        3
-
         """
         return len(self._cache)
 
@@ -296,12 +303,12 @@ class Deque(Sequence):
 
         >>> deque = Deque(directory='/tmp/diskcache/deque')
         >>> deque.clear()
-        >>> deque.extend('abc')
+        >>> deque.extend('abcd')
         >>> iterator = reversed(deque)
         >>> next(iterator)
-        'c'
+        'd'
         >>> list(iterator)
-        ['b', 'a']
+        ['c', 'b', 'a']
 
         """
         _cache = self._cache
@@ -372,15 +379,6 @@ class Deque(Sequence):
     def clear(self):
         """Remove all elements from deque.
 
-        >>> deque = Deque(directory='/tmp/diskcache/deque')
-        >>> deque.clear()
-        >>> deque += range(10)
-        >>> len(deque)
-        10
-        >>> deque.clear()
-        >>> len(deque)
-        0
-
         """
         _cache_clear = self._cache.clear
 
@@ -414,12 +412,6 @@ class Deque(Sequence):
     def extend(self, iterable):
         """Extend back side of deque with values from `iterable`.
 
-        >>> deque = Deque(directory='/tmp/diskcache/deque')
-        >>> deque.clear()
-        >>> deque.extend('abc')
-        >>> list(deque)
-        ['a', 'b', 'c']
-
         :param iterable: iterable of values
 
         """
@@ -532,7 +524,13 @@ class Deque(Sequence):
 
         for key in _cache.iterkeys():
             try:
-                item = _cache[key]
+                while True:
+                    try:
+                        item = _cache[key]
+                    except Timeout:
+                        continue
+                    else:
+                        break
             except KeyError:
                 continue
             else:
@@ -554,14 +552,8 @@ class Deque(Sequence):
     def reverse(self):
         """Reverse deque in place.
 
-        >>> deque = Deque(directory='/tmp/diskcache/deque')
-        >>> deque.clear()
-        >>> deque.extend('abc')
-        >>> deque.reverse()
-        >>> list(deque)
-        ['c', 'b', 'a']
-
         """
+        # pylint: disable=protected-access
         directory = mkdtemp()
         temp = None
 
@@ -570,6 +562,7 @@ class Deque(Sequence):
             self.clear()
             self.extend(temp)
         finally:
+            temp._cache.close()
             del temp
             rmtree(directory)
 
@@ -596,7 +589,14 @@ class Deque(Sequence):
             type_name = type(steps).__name__
             raise TypeError('integer argument expected, got %s' % type_name)
 
+        len_self = len(self)
+
+        if not len_self:
+            return
+
         if steps >= 0:
+            steps %= len_self
+
             for _ in range(steps):
                 try:
                     value = self.pop()
@@ -605,7 +605,10 @@ class Deque(Sequence):
                 else:
                     self.appendleft(value)
         else:
-            for _ in range(-steps):
+            steps *= -1
+            steps %= len_self
+
+            for _ in range(steps):
                 try:
                     value = self.popleft()
                 except IndexError:
@@ -614,6 +617,10 @@ class Deque(Sequence):
                     self.append(value)
 
 
+    def __del__(self):
+        self._cache.close()
+
+
     __hash__ = None
 
 
@@ -626,16 +633,20 @@ class Index(MutableMapping):
     Hashing protocol is not used. Keys are looked up by their serialized
     format. See ``diskcache.Disk`` for details.
 
-    >>> index = Index([('apple', 1), ('beans', 2), ('cherries', 3)])
-    >>> index['apple']
+    >>> index = Index('/tmp/diskcache/index')
+    >>> index
+    Index('/tmp/diskcache/index')
+    >>> index.clear()
+    >>> index.update([('a', 1), ('b', 2), ('c', 3)])
+    >>> index['a']
     1
     >>> list(index)
-    ['apple', 'beans', 'cherries']
+    ['a', 'b', 'c']
     >>> len(index)
     3
-    >>> del index['beans']
+    >>> del index['b']
     >>> index.popitem()
-    ('cherries', 3)
+    ('c', 3)
 
     """
     def __init__(self, *args, **kwargs):
@@ -644,12 +655,12 @@ class Index(MutableMapping):
         Optional first argument may be string specifying directory where items
         are stored. When None or not given, temporary directory is created.
 
-        >>> index = Index({'apples': 1, 'beans': 2, 'cherries': 3})
+        >>> index = Index({'a': 1, 'b': 2, 'c': 3})
         >>> len(index)
         3
         >>> directory = index.directory
-        >>> inventory = Index(directory, dates=4)
-        >>> inventory['beans']
+        >>> inventory = Index(directory, d=4)
+        >>> inventory['b']
         2
         >>> len(inventory)
         4
@@ -662,10 +673,37 @@ class Index(MutableMapping):
             if args and args[0] is None:
                 args = args[1:]
             directory = mkdtemp(prefix='diskcache-')
-        self._cache = Cache(directory, eviction_policy='none', timeout=0.001)
+        self._cache = Cache(directory, eviction_policy='none')
         self.update(*args, **kwargs)
 
 
+    @classmethod
+    def fromcache(cls, cache, *args, **kwargs):
+        """Initialize index using `cache` and update items.
+
+        >>> cache = Cache('/tmp/diskcache/index')
+        >>> _ = cache.clear()
+        >>> index = Index.fromcache(cache, {'a': 1, 'b': 2, 'c': 3})
+        >>> len(index)
+        3
+        >>> 'b' in index
+        True
+        >>> index['c']
+        3
+
+        :param Cache cache: cache to use
+        :param args: mapping or sequence of items
+        :param kwargs: mapping of items
+        :return: initialized Index
+
+        """
+        # pylint: disable=no-member,protected-access
+        self = cls.__new__(cls)
+        self._cache = cache
+        self.update(*args, **kwargs)
+        return self
+
+
     @property
     def directory(self):
         "Directory path where items are stored."
@@ -679,11 +717,11 @@ class Index(MutableMapping):
 
         >>> index = Index('/tmp/diskcache/index')
         >>> index.clear()
-        >>> index.update({'a': 0, 'b': 1})
+        >>> index.update({'a': 1, 'b': 2})
         >>> index['a']
-        0
-        >>> index['b']
         1
+        >>> index['b']
+        2
         >>> index['c']
         Traceback (most recent call last):
             ...
@@ -709,7 +747,7 @@ class Index(MutableMapping):
 
         >>> index = Index('/tmp/diskcache/index')
         >>> index.clear()
-        >>> index['a'] = 0
+        >>> index['a'] = 1
         >>> index[0] = None
         >>> len(index)
         2
@@ -736,7 +774,7 @@ class Index(MutableMapping):
 
         >>> index = Index('/tmp/diskcache/index')
         >>> index.clear()
-        >>> index.update({'a': 0, 'b': 1})
+        >>> index.update({'a': 1, 'b': 2})
         >>> del index['a']
         >>> del index['b']
         >>> len(index)
@@ -767,13 +805,13 @@ class Index(MutableMapping):
         If `key` is missing then return `default`. If `default` is `ENOVAL`
         then raise KeyError.
 
-        >>> index = Index('/tmp/diskcache/index', {'a': 0, 'b': 1})
+        >>> index = Index('/tmp/diskcache/index', {'a': 1, 'b': 2})
         >>> index.pop('a')
-        0
-        >>> index.pop('b')
         1
-        >>> index.pop('c', default=2)
+        >>> index.pop('b')
         2
+        >>> index.pop('c', default=3)
+        3
         >>> index.pop('d')
         Traceback (most recent call last):
             ...
@@ -808,13 +846,13 @@ class Index(MutableMapping):
 
         >>> index = Index('/tmp/diskcache/index')
         >>> index.clear()
-        >>> index.update([('apples', 1), ('beans', 2), ('cherries', 3)])
+        >>> index.update([('a', 1), ('b', 2), ('c', 3)])
         >>> index.popitem()
-        ('cherries', 3)
+        ('c', 3)
+        >>> index.popitem(last=False)
+        ('a', 1)
         >>> index.popitem()
-        ('beans', 2)
-        >>> index.popitem()
-        ('apples', 1)
+        ('b', 2)
         >>> index.popitem()
         Traceback (most recent call last):
           ...
@@ -858,11 +896,11 @@ class Index(MutableMapping):
 
         >>> index = Index('/tmp/diskcache/index')
         >>> index.clear()
-        >>> index.push('apples')
+        >>> print(index.push('apples'))
         500000000000000
-        >>> index.push('beans')
+        >>> print(index.push('beans'))
         500000000000001
-        >>> index.push('cherries', side='front')
+        >>> print(index.push('cherries', side='front'))
         499999999999999
         >>> index[500000000000001]
         'beans'
@@ -901,14 +939,18 @@ class Index(MutableMapping):
         >>> index = Index('/tmp/diskcache/index')
         >>> index.clear()
         >>> for letter in 'abc':
-        ...     index.push(letter)
+        ...     print(index.push(letter))
         500000000000000
         500000000000001
         500000000000002
-        >>> index.pull()
-        (500000000000000, 'a')
-        >>> index.pull(side='back')
-        (500000000000002, 'c')
+        >>> key, value = index.pull()
+        >>> print(key)
+        500000000000000
+        >>> value
+        'a'
+        >>> _, value = index.pull(side='back')
+        >>> value
+        'c'
         >>> index.pull(prefix='fruit')
         (None, None)
 
@@ -931,15 +973,6 @@ class Index(MutableMapping):
     def clear(self):
         """Remove all items from index.
 
-        >>> index = Index('/tmp/diskcache/index')
-        >>> index.clear()
-        >>> index.update({'a': 0, 'b': 1, 'c': 2})
-        >>> len(index)
-        3
-        >>> index.clear()
-        >>> len(index)
-        0
-
         """
         _cache_clear = self._cache.clear
 
@@ -956,15 +989,6 @@ class Index(MutableMapping):
 
         Return iterator of index keys in insertion order.
 
-        >>> index = Index('/tmp/diskcache/index')
-        >>> index.clear()
-        >>> index.update([('a', 0), ('b', 1), ('c', 2)])
-        >>> iterator = iter(index)
-        >>> next(iterator)
-        'a'
-        >>> list(iterator)
-        ['b', 'c']
-
         """
         return iter(self._cache)
 
@@ -976,7 +1000,7 @@ class Index(MutableMapping):
 
         >>> index = Index('/tmp/diskcache/index')
         >>> index.clear()
-        >>> index.update([('a', 0), ('b', 1), ('c', 2)])
+        >>> index.update([('a', 1), ('b', 2), ('c', 3)])
         >>> iterator = reversed(index)
         >>> next(iterator)
         'c'
@@ -992,18 +1016,220 @@ class Index(MutableMapping):
 
         Return length of index.
 
-        >>> index = Index('/tmp/diskcache/index')
-        >>> index.clear()
-        >>> len(index)
-        0
-        >>> index.update({'a': 0, 'b': 1, 'c': 2})
-        >>> len(index)
-        3
-
         """
         return len(self._cache)
 
 
+    if sys.hexversion < 0x03000000:
+        def keys(self):
+            """List of index keys.
+
+            >>> index = Index('/tmp/diskcache/index')
+            >>> index.clear()
+            >>> index.update([('a', 1), ('b', 2), ('c', 3)])
+            >>> index.keys()
+            ['a', 'b', 'c']
+
+            :return: list of keys
+
+            """
+            return list(self._cache)
+
+
+        def values(self):
+            """List of index values.
+
+            >>> index = Index('/tmp/diskcache/index')
+            >>> index.clear()
+            >>> index.update([('a', 1), ('b', 2), ('c', 3)])
+            >>> index.values()
+            [1, 2, 3]
+
+            :return: list of values
+
+            """
+            return list(self.itervalues())
+
+
+        def items(self):
+            """List of index items.
+
+            >>> index = Index('/tmp/diskcache/index')
+            >>> index.clear()
+            >>> index.update([('a', 1), ('b', 2), ('c', 3)])
+            >>> index.items()
+            [('a', 1), ('b', 2), ('c', 3)]
+
+            :return: list of items
+
+            """
+            return list(self.iteritems())
+
+
+        def iterkeys(self):
+            """Iterator of index keys.
+
+            >>> index = Index('/tmp/diskcache/index')
+            >>> index.clear()
+            >>> index.update([('a', 1), ('b', 2), ('c', 3)])
+            >>> list(index.iterkeys())
+            ['a', 'b', 'c']
+
+            :return: iterator of keys
+
+            """
+            return iter(self._cache)
+
+
+        def itervalues(self):
+            """Iterator of index values.
+
+            >>> index = Index('/tmp/diskcache/index')
+            >>> index.clear()
+            >>> index.update([('a', 1), ('b', 2), ('c', 3)])
+            >>> list(index.itervalues())
+            [1, 2, 3]
+
+            :return: iterator of values
+
+            """
+            _cache = self._cache
+
+            for key in _cache:
+                while True:
+                    try:
+                        yield _cache[key]
+                    except KeyError:
+                        break
+                    except Timeout:
+                        continue
+                    else:
+                        break
+
+
+        def iteritems(self):
+            """Iterator of index items.
+
+            >>> index = Index('/tmp/diskcache/index')
+            >>> index.clear()
+            >>> index.update([('a', 1), ('b', 2), ('c', 3)])
+            >>> list(index.iteritems())
+            [('a', 1), ('b', 2), ('c', 3)]
+
+            :return: iterator of items
+
+            """
+            _cache = self._cache
+
+            for key in _cache:
+                while True:
+                    try:
+                        yield key, _cache[key]
+                    except KeyError:
+                        break
+                    except Timeout:
+                        continue
+                    else:
+                        break
+
+
+        def viewkeys(self):
+            """Set-like object providing a view of index keys.
+
+            >>> index = Index('/tmp/diskcache/index')
+            >>> index.clear()
+            >>> index.update({'a': 1, 'b': 2, 'c': 3})
+            >>> keys_view = index.viewkeys()
+            >>> 'b' in keys_view
+            True
+
+            :return: keys view
+
+            """
+            return KeysView(self)
+
+
+        def viewvalues(self):
+            """Set-like object providing a view of index values.
+
+            >>> index = Index('/tmp/diskcache/index')
+            >>> index.clear()
+            >>> index.update({'a': 1, 'b': 2, 'c': 3})
+            >>> values_view = index.viewvalues()
+            >>> 2 in values_view
+            True
+
+            :return: values view
+
+            """
+            return ValuesView(self)
+
+
+        def viewitems(self):
+            """Set-like object providing a view of index items.
+
+            >>> index = Index('/tmp/diskcache/index')
+            >>> index.clear()
+            >>> index.update({'a': 1, 'b': 2, 'c': 3})
+            >>> items_view = index.viewitems()
+            >>> ('b', 2) in items_view
+            True
+
+            :return: items view
+
+            """
+            return ItemsView(self)
+
+
+    else:
+        def keys(self):
+            """Set-like object providing a view of index keys.
+
+            >>> index = Index('/tmp/diskcache/index')
+            >>> index.clear()
+            >>> index.update({'a': 1, 'b': 2, 'c': 3})
+            >>> keys_view = index.keys()
+            >>> 'b' in keys_view
+            True
+
+            :return: keys view
+
+            """
+            return KeysView(self)
+
+
+        def values(self):
+            """Set-like object providing a view of index values.
+
+            >>> index = Index('/tmp/diskcache/index')
+            >>> index.clear()
+            >>> index.update({'a': 1, 'b': 2, 'c': 3})
+            >>> values_view = index.values()
+            >>> 2 in values_view
+            True
+
+            :return: values view
+
+            """
+            return ValuesView(self)
+
+
+        def items(self):
+            """Set-like object providing a view of index items.
+
+            >>> index = Index('/tmp/diskcache/index')
+            >>> index.clear()
+            >>> index.update({'a': 1, 'b': 2, 'c': 3})
+            >>> items_view = index.items()
+            >>> ('b', 2) in items_view
+            True
+
+            :return: items view
+
+            """
+            return ItemsView(self)
+
+
     __hash__ = None
 
 
@@ -1025,13 +1251,13 @@ class Index(MutableMapping):
 
         >>> index = Index('/tmp/diskcache/index')
         >>> index.clear()
-        >>> pairs = [('a', 0), ('b', 1), ('c', 2)]
+        >>> pairs = [('a', 1), ('b', 2), ('c', 3)]
         >>> index.update(pairs)
         >>> from collections import OrderedDict
         >>> od = OrderedDict(pairs)
         >>> index == od
         True
-        >>> index == {'c': 2, 'b': 1, 'a': 0}
+        >>> index == {'c': 3, 'b': 2, 'a': 1}
         True
 
         :param other: other mapping in equality comparison
@@ -1059,12 +1285,12 @@ class Index(MutableMapping):
 
         >>> index = Index('/tmp/diskcache/index')
         >>> index.clear()
-        >>> index.update([('a', 0), ('b', 1), ('c', 2)])
+        >>> index.update([('a', 1), ('b', 2), ('c', 3)])
         >>> from collections import OrderedDict
-        >>> od = OrderedDict([('c', 2), ('b', 1), ('a', 0)])
+        >>> od = OrderedDict([('c', 3), ('b', 2), ('a', 1)])
         >>> index != od
         True
-        >>> index != {'a': 0, 'b': 1}
+        >>> index != {'a': 1, 'b': 2}
         True
 
         :param other: other mapping in inequality comparison
@@ -1078,10 +1304,10 @@ class Index(MutableMapping):
 
         Return string with printable representation of index.
 
-        >>> index = Index('/tmp/diskcache/index')
-        >>> index
-        Index('/tmp/diskcache/index')
-
         """
         name = type(self).__name__
         return '{0}({1!r})'.format(name, self.directory)
+
+
+    def __del__(self):
+        self._cache.close()
